/* 
 *	Functions related to command completion
 */

#include "launch.h"
#include <dirent.h>
#include <sys/types.h>

int CmpVariants (void);

/*
 * Function return found variants & number of variants if present
 *
 * 0 - directory containe only one filename
 * 1 - directory doesn`t contain any filename
 * 2 - directory contain someone filenames
 * 3 - reserved
 * 4 - error
 *
 */
int
makeCompletion (void)
{
  DIR           *dir;
  struct stat   filestatus;
  struct dirent	*dirbuf;
  char          *work_text, *test_dir, *test_name, *tmp_text, *tmp_path;
  char          *search_path[32];
  int           npath, i, j, tmplen;
  int           testEnl = 0, isDir = 0;

  nvariants = 0;
  complete_text[0] = '\0';
  leader_text[0] = '\0';
	
  /* Textfield empty */
  if (!strlen (WMGetTextFieldText (LaunchPanel->command_tf)))
	return (4);

  work_text =
	(char *) malloc (strlen (WMGetTextFieldText (LaunchPanel->command_tf)) +
					 strlen (home_var) + 2);
  strcpy (work_text, WMGetTextFieldText (LaunchPanel->command_tf));

  tmp_text = strrchr(work_text, ' ');		// search last space
  if (tmp_text != NULL)						// is the text to complete parameter
	{
	  strcpy (leader_text, work_text);
	  if (tmp_text[0] != '\0')
		leader_text[ strlen (work_text) - strlen (tmp_text) + 1] = '\0';
	  strcpy (work_text, tmp_text + 1);
	}
  else
	{
	  testEnl = 1;
	}

  tmp_text = strrchr(work_text, '/');		// search last slash
  if (tmp_text != NULL)						// is the text to complete directory
	{
	  strcpy (complete_text, work_text);
	  if (tmp_text[0] != '\0')
		complete_text[ strlen (work_text) - strlen (tmp_text) + 1] = '\0';
	  strcpy (work_text, tmp_text + 1);
	  isDir = 1;
	}
  else
	{
	  if (!strncmp(work_text,"~",1))
		{
		  strcpy(complete_text, "~");
		  strcpy(work_text,work_text+1);
		  isDir = 1;
		}
	}

  test_dir = (char *) malloc (strlen(complete_text) + strlen(home_var) + 2);
  if(!strncmp(complete_text,"~",1))
	{
	  strcpy (test_dir, home_var);
	  strcat (test_dir, "/");
	  strcat (test_dir, complete_text + 1);
	}
  else
	{
	  strcpy (test_dir, complete_text);
	}

  /* Input text is a regular FILE, search by $PATHs */
	
  if (!isDir)
	{
	  /* Is Variable $PATH empty ? */
	  if (strlen (path) == 0)
		{
		  free (work_text);
		  return (1);
		}
	  tmp_path = (char *) malloc (strlen (path) + 1);
	  strcpy (tmp_path, path);
	  i = 0;
	  npath = 0;
	  search_path[0] = tmp_path;

	  /* Replace ':' on '\0' */
	  while (path[i])
		{
		  if (tmp_path[i] == ':')
			tmp_path[i] = '\0';
		  i++;
		}
	  i = 0;
		
	  /* Create PATH array */
	  while (path[i])
		{
		  if (!tmp_path[i])
			{
			  npath++;
			  search_path[npath] = tmp_path + i + 1;
			  /* Search double PATH */
			  for (j = 0; j < npath; j++)
				{
				  if (!strcmp (search_path[j], search_path[npath]))
					{
					  npath--;
					  continue;
					}
				}
			}
		  i++;
		}

	  /* Search programs in PATH */
	  for (i = 0; i <= npath; i++)
		{
		  if ( (dir = opendir(search_path[i])) )
			{
			  while ( (dirbuf = readdir(dir)) )
				{
				  if (!strncmp (work_text, dirbuf->d_name, strlen (work_text)))
					{
					  test_name = (char *) malloc (strlen (dirbuf->d_name) +
												   strlen (search_path[i]) + 2);
					  strcpy (test_name, search_path[i]);
					  strcpy (test_name + strlen (search_path[i]), "/");
					  strcpy (test_name + strlen (search_path[i]) + 1, dirbuf->d_name);
						
					  if (!stat (test_name, &filestatus) 
						  && (filestatus.st_mode & (S_IXOTH | S_IXGRP | S_IXUSR))
						  && !(filestatus.st_mode & S_IFDIR))
						{
						  variants[nvariants] = 
							(char *) malloc (strlen (dirbuf->d_name) + 2);
						  strcpy (variants[nvariants], dirbuf->d_name);
						  nvariants++;
						}
					  free (test_name);
					}
				  /* Too many variants */
				  if (nvariants == 254)
					{
					  closedir (dir);
					  free (tmp_path);
					  free (test_dir);
					  free (work_text);
					  return (4);
					}
				}
			  closedir (dir);
			}
		}
	  free (tmp_path);
	  free (test_dir);
	  free (work_text);
	  /* Directories doesn`t contain any file */
	  if (!nvariants)
		return (1);
	  else
		/* Directories contains only one file */
		if (nvariants == 1)
		  return (0);
		else
		  {
			tmplen = CmpVariants ();
			if (tmplen)
			  {
				variants[0][tmplen] = '\0';
				return (0);
			  }
			/* Directories contains someone files */
			return (2);
		  }
	}


  /*	Input text is a DIRECTORY, search by text */
  else
	{
	  if ( (dir = opendir(test_dir)) )
		{
		  while ( (dirbuf = readdir(dir)) )
			{
			  if (!strncmp (work_text, dirbuf->d_name, 
							strlen (work_text)))
				{
				  variants[nvariants] = (char *) malloc (strlen (dirbuf->d_name) + 2);
						
				  strcpy (variants[nvariants], dirbuf->d_name);
				  tmp_text = (char *) malloc (strlen (variants[nvariants]) + strlen(test_dir) + 2);
				  strcpy (tmp_text, test_dir);
				  strcat (tmp_text, variants[nvariants]);
				  if (!stat (tmp_text, &filestatus)	//Directory
					  && (filestatus.st_mode & (S_IXOTH | S_IXGRP | S_IXUSR))
					  && (filestatus.st_mode & S_IFDIR))
					{
					  if (!strncmp (variants[nvariants], ".", 1))
						{
						  if(strlen(variants[nvariants])==1)
							free (variants[nvariants]);
						  else
							{
							  if(strlen(variants[nvariants])==2 && !strncmp (variants[nvariants]+1, ".", 1))
								free (variants[nvariants]);
							  else
								{
								  strcat (variants[nvariants], "/");
								  nvariants++;
								}
							}
						}
					  else
						{
						  strcat (variants[nvariants], "/");
						  nvariants++;
						}
					}
				  else if (!stat (tmp_text, &filestatus)	//Regular execute file
						   && ((filestatus.st_mode & (S_IXOTH | S_IXGRP | S_IXUSR)) | !testEnl)
						   && !(filestatus.st_mode & S_IFDIR))
					{
					  nvariants++;
					}
				  else
					{
					  free (variants[nvariants]);
					}
				  free (tmp_text);
				}
			  if (nvariants == 254)	//Too many variants
				{
				  closedir (dir);
				  free (test_dir);
				  free (work_text);
				  return (4);
				}
			}
		  closedir (dir);
		  free (test_dir);
		  free (work_text);
		  if (!nvariants)
			return (1);		//Directory isn`t contein anyone file
		  else if (nvariants == 1)
			return (0);		//Directory is contain only one file
		  else
			{
			  tmplen = CmpVariants ();
			  if (tmplen)
				{
				  variants[0][tmplen] = '\0';
				  return (0);
				}
			  return (2);	//Directory is contain more then one file
			}
		}
	  else			//Invalid directory
		{
		  test_dir = (char *) malloc (strlen(complete_text));
		  strcpy(test_dir, complete_text);
		  test_dir[strlen(complete_text)-1] = '\0';
		  while ( (tmp_text = strrchr(test_dir, '/')) )	//Search valid directory
			{
			  tmp_text[1] = '\0';
			  if ( (dir = opendir(test_dir)) )
				{
				  variants[0] = (char *) malloc (strlen (test_dir) + 1);
				  strcpy (variants[0], test_dir);
				  nvariants = 1;
				  closedir (dir);
				  free (test_dir);
				  free (work_text);
				  return (3);	//Valid directory
				}
			  tmp_text[0] = '\0';
			}
		  free (work_text);
		  free (test_dir);
		  return (1);
		}
	}
}



/*
 *	Function search the similiar part of variants 
 *  and return number of char in this part
 */
int
CmpVariants (void)
{
  int		nvar, textlen, minlen, i, cmp_ok;

  nvar = nvariants;
  textlen = strlen (WMGetTextFieldText (LaunchPanel->command_tf));
  minlen = strlen (variants[0]);
  for (i = 2; nvar > i; i++)
	if (strlen (variants[i - 1]) < minlen)
	  minlen = strlen (variants[i - 1]);
  while (minlen > textlen)
	{
	  cmp_ok = 1;
	  for (i = 2; nvar >= i; i++)
		if (strncmp (variants[0], variants[i - 1], minlen))
		  cmp_ok = 0;
	  if (cmp_ok)
		return (minlen);
	  minlen--;
	}
  return (0);			// No simple variants
}
