/* 
 *	"Command Execution" panel.
 *	 runExecPanel() - adds widgets to common panel specific to this one.
 */

#include "launch.h"
#include <ctype.h>

static void UsernamePressAction (XEvent * event, WMWidget *widget);
static void TextfieldPressAction (XEvent * event, WMWidget *widget);
static void VarlistPressAction (XEvent * event, WMWidget *widget);
static void ListEnteredAction (void);
static void EnteredAction (void);

/***************************** Keys press actions ****************************/

/*
 *	Key pressed in "Username" textfield
 */
static void
UsernamePressAction (XEvent * event, WMWidget *widget)
{
  char	*key_pressed;

  key_pressed = KeyPressAction(event, widget);
	
  if (!strcmp (key_pressed, "Tab"))
	{
	  WMSetFocusToWidget (LaunchPanel->command_tf);
	}
  if (!strcmp (key_pressed, "Return") || !strcmp (key_pressed, "KP_Enter"))
	{
	  WMSetFocusToWidget (LaunchPanel->command_tf);
	}

  return;
}

/* 
 *	Key pressed in "Command" textfield 
 */
static void
TextfieldPressAction (XEvent * event, WMWidget *widget)
{
  char	*key_pressed, *text_field;
  int		i;

  key_pressed = KeyPressAction(event, widget);
  
  if (strlen (key_pressed) == 1 && isalnum(key_pressed[0]))
	{
	  for (i = 0; i < commands_count; i++)
		{
		  text_field = WMGetTextFieldText (LaunchPanel->command_tf);
		  if (!strncmp (command_history[i], text_field, strlen(text_field)))
			{
			  current_command = i;
			  WMSetTextFieldText (LaunchPanel->command_tf, command_history[i]);
			  LaunchPanel->text_range = wmkrange (strlen (command_history[i]),
												  strlen (text_field) - strlen (command_history[i]));
			  WMSelectTextFieldRange (LaunchPanel->command_tf, 
									  LaunchPanel->text_range);
			  break;
			}
		}
	}

  if (!strcmp (key_pressed, "Up"))
	{
	  if (current_command == 0)
		{
		  XBell (display, 100);
		  return;
		}
	  if (current_command == commands_count)
		{
		  strcpy (command_history[current_command],
				  WMGetTextFieldText (LaunchPanel->command_tf));
		}
	  current_command--;
	  WMSetTextFieldText (LaunchPanel->command_tf, 
						  command_history[current_command]);
	}
	
  if (!strcmp (key_pressed, "Down"))
	{
	  if (current_command >= commands_count)
		{
		  XBell (display, 100);
		  return;
		}
	  current_command++;
	  WMSetTextFieldText (LaunchPanel->command_tf, 
						  command_history[current_command]);
	}
	
  if (!strcmp (key_pressed, "TabShifted"))
	{
	  WMSetFocusToWidget (LaunchPanel->username_tf);
	}

  if (!strcmp (key_pressed, "Tab"))
	{
	  switch (makeCompletion ())
		{
		case 0:
//				if (leader_text)
//				{
		  leader_tmp = strcat (leader_text, complete_text);
		  leader_tmp = strcat (leader_text, variants[0]);
		  WMSetTextFieldText (LaunchPanel->command_tf, leader_tmp);
//				}
//				else
//					WMSetTextFieldText (LaunchPanel->command_tf, variants[0]);
		  break;
		case 1:
		  XBell (display, 100);
		  WMSetLabelText (LaunchPanel->info, "No such file or directory!");
		  break;
		case 2:
		  if (!tabkeypress)
			{
			  XBell (display, 100);
			  WMSetLabelText (LaunchPanel->info, "Some variants found!");
			  tabkeypress = 1;
			}
		  else
			{
			  /* main panel */
			  WMResizeWidget (LaunchPanel->window, 324, 214 +
							  WMFontHeight ((wm_screen->normalFont)) * 7 + 22);
						
			  /* buttons widgets */
			  WMMoveWidget (LaunchPanel->run_button, 240, 202	+ 
							WMFontHeight ((wm_screen->normalFont)) * 7 + 2);
			  WMMoveWidget (LaunchPanel->cancel_button, 156, 202 +
							WMFontHeight ((wm_screen->normalFont)) * 7 + 2);
			  if (WMGetUDBoolForKey (udb, "Enable_Switch"))
				{
				  WMMoveWidget (LaunchPanel->switch_button, 12, 202 +
								WMFontHeight ((wm_screen->normalFont)) * 7 + 2);
				}
						
			  /* create list */
			  WMClearList (LaunchPanel->varlist);
			  for (i = 0; i < nvariants; i++)
				{
				  WMInsertListItem (LaunchPanel->varlist, i, variants[i]);
				}
						
			  WMSortListItems (LaunchPanel->varlist);
			  WMSelectListItem (LaunchPanel->varlist, 0);
			  WMRealizeWidget (LaunchPanel->window);
			  WMMapWidget (LaunchPanel->varlist);
			  WMSetFocusToWidget (LaunchPanel->varlist);
					
			  tabkeypress = 0;
			}
		  break;
		case 3:
		  XBell (display, 100);
		  WMSetLabelText (LaunchPanel->info, "Invalid pathname!");
		  WMSetTextFieldCursorPosition (LaunchPanel->command_tf, 
										strlen (variants[0]));
		  LaunchPanel->text_range = wmkrange (strlen (variants[0]),
											  strlen (WMGetTextFieldText (LaunchPanel->command_tf)) -
											  strlen (variants[0]));
		  WMSelectTextFieldRange (LaunchPanel->command_tf, 
								  LaunchPanel->text_range);
		  break;
		case 4:
		  XBell (display, 100);
		  WMSetLabelText (LaunchPanel->info, "Too many variants!");
		  break;
		}
	  if (nvariants)
		while (nvariants--)
		  free (variants[nvariants]);
	}
		
  if (!strcmp (key_pressed, "Return") || !strcmp (key_pressed, "KP_Enter"))
	{
	  EnteredAction ();
	}
		
  return;
}

/*
 *	Key pressed in variants list
 */
static void
VarlistPressAction (XEvent * event, WMWidget *widget)
{
  char	*key_pressed;

  key_pressed = KeyPressAction(event, widget);

  if (!strcmp (key_pressed, "Up"))
	{
	  if (WMGetListSelectedItemRow (LaunchPanel->varlist))
		{
		  WMSelectListItem (LaunchPanel->varlist,
							WMGetListSelectedItemRow (LaunchPanel->varlist) -	1);
		  if (WMGetListSelectedItemRow (LaunchPanel->varlist) <
			  WMGetListPosition (LaunchPanel->varlist))
			{
			  WMSetListPosition (LaunchPanel->varlist,
								 WMGetListPosition (LaunchPanel->varlist) - 1);
			}
		}
	}

  if (!strcmp (key_pressed, "Down"))
	{
	  if (WMGetListSelectedItemRow (LaunchPanel->varlist) != nvariants)
		{
		  WMSelectListItem (LaunchPanel->varlist,
							WMGetListSelectedItemRow (LaunchPanel->varlist) +	1);
		  if (WMGetListSelectedItemRow (LaunchPanel->varlist) >
			  WMGetListPosition (LaunchPanel->varlist) + 6)
			{
			  WMSetListPosition (LaunchPanel->varlist,
								 WMGetListPosition (LaunchPanel->varlist) + 1);
			}
		}
	}

  if (!strcmp (key_pressed, "Return") || !strcmp (key_pressed, "KP_Enter"))
	{
	  ListEnteredAction ();
	}

  if (!strcmp (key_pressed, "Escape"))
	{
	  WMResizeWidget (LaunchPanel->window, 324, 234);
	  WMMoveWidget (LaunchPanel->run_button, 240, 202);
	  WMMoveWidget (LaunchPanel->cancel_button, 156, 202);
	  if (WMGetUDBoolForKey (udb, "Enable_Switch"))
		{
		  WMMoveWidget (LaunchPanel->switch_button, 12, 206);
		}
	  WMRealizeWidget (LaunchPanel->window);
	  WMUnmapWidget (LaunchPanel->varlist);
	  WMSetFocusToWidget (LaunchPanel->command_tf);
	}

  return;
}

/**************************** Widgets actions ********************************/

/* 
 *	Pressed Enter or double clicked variants list 
 */
static void
ListEnteredAction (void)
{
  WMUnmapWidget (LaunchPanel->varlist);
  if (leader_text)
	{
	  leader_tmp = strcat (leader_text, complete_text);
	  leader_tmp = strcat (leader_text, WMGetListSelectedItem (LaunchPanel->varlist)->text);

	  WMSetTextFieldText (LaunchPanel->command_tf, leader_tmp);
	}
  else
	WMSetTextFieldText (LaunchPanel->command_tf,
						WMGetListSelectedItem (LaunchPanel->varlist)->text);

  WMResizeWidget (LaunchPanel->window, 324, 234);
  WMMoveWidget (LaunchPanel->run_button, 240, 202);
  WMMoveWidget (LaunchPanel->cancel_button, 156, 202);
  if (WMGetUDBoolForKey (udb, "Enable_Switch"))
	{
	  WMMoveWidget (LaunchPanel->switch_button, 12, 206);
	}
  WMRealizeWidget (LaunchPanel->window);
  WMSetFocusToWidget (LaunchPanel->command_tf);
  return;
}

/* 
 *	Let's execute!
 */
static void
EnteredAction (void)
{
  if (WMGetUDBoolForKey (udb, "Enable_Switch"))
	ConsoleApp = WMGetButtonSelected (LaunchPanel->switch_button);

  strcpy (entered_text, WMGetTextFieldText (LaunchPanel->command_tf));
  strcpy (entered_user, WMGetTextFieldText (LaunchPanel->username_tf));
  pw = getpwnam (entered_user);
  if (strcmp (entered_user, getenv ("USER")) != 0
	  && strcmp (entered_text, ""))
	{
	  auth = 1;
	  runAuthPanel ();
	  WMMapWidget (LaunchPanel->window);
	  return;
	}
  LaunchCloseAction ();
}

/*****************************************************************************/
void
runExecPanel (void)
{
  /* title label */
  WMSetLabelText (LaunchPanel->title, "Command Execution");

  /* info label */
  WMSetLabelText (LaunchPanel->info, "Use Up and Down keys to browse history.");

  /* username label */
  LaunchPanel->username_prompt = WMCreateLabel (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->username_prompt, 324, 12);
  WMMoveWidget (LaunchPanel->username_prompt, 12, 131);
  WMSetLabelTextAlignment (LaunchPanel->username_prompt, WALeft);
  WMSetLabelText (LaunchPanel->username_prompt, "Username:");

  /* textfield widget(username) */
  LaunchPanel->text_range = wmkrange (0, strlen (getenv ("USER")));
  LaunchPanel->username_tf = WMCreateTextField (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->username_tf, 240, 20);
  WMMoveWidget (LaunchPanel->username_tf, 76, 127);
  WMSetTextFieldText (LaunchPanel->username_tf, getenv ("USER"));
  WMSetTextFieldEditable (LaunchPanel->username_tf, True);
  WMSelectTextFieldRange (LaunchPanel->username_tf, LaunchPanel->text_range);

  /* command label */
  LaunchPanel->command_prompt = WMCreateLabel (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->command_prompt, 324, 12);
  WMMoveWidget (LaunchPanel->command_prompt, 12, 156);
  WMSetLabelTextAlignment (LaunchPanel->command_prompt, WALeft);
  WMSetLabelText (LaunchPanel->command_prompt, "Command:");

  /* textfield widget(command) */
  LaunchPanel->command_tf = WMCreateTextField (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->command_tf, 240, 20);
  WMMoveWidget (LaunchPanel->command_tf, 76, 154);

  /* check button widget */
  if (WMGetUDBoolForKey (udb, "Enable_Switch"))
	{
	  LaunchPanel->switch_button = WMCreateSwitchButton (LaunchPanel->window);
	  WMResizeWidget (LaunchPanel->switch_button, 140, 20);
	  WMMoveWidget (LaunchPanel->switch_button, 12, 206);
	  WMSetButtonText (LaunchPanel->switch_button, "Console application");
	  WMSetButtonSelected (LaunchPanel->switch_button, False);
	  ConsoleSwitchVisible = 1;
	}

  /* buttons widgets */

  /* "Run" */
  LaunchPanel->run_button = WMCreateCommandButton (LaunchPanel->window);
  WMSetButtonAction (LaunchPanel->run_button, (WMAction *) EnteredAction, NULL);
  WMResizeWidget (LaunchPanel->run_button, 76, 24);
  WMMoveWidget (LaunchPanel->run_button, 240, 202);
  WMSetButtonText (LaunchPanel->run_button, "Execute");
  WMSetButtonImage (LaunchPanel->run_button, WMGetSystemPixmap (wm_screen,
																WSIReturnArrow));
  WMSetButtonAltImage (LaunchPanel->run_button, WMGetSystemPixmap (wm_screen,
																   WSIHighlightedReturnArrow));
  WMSetButtonImagePosition (LaunchPanel->run_button, WIPRight);

  /* "Cancel" */
  LaunchPanel->cancel_button = WMCreateCommandButton (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->cancel_button, 76, 24);
  WMMoveWidget (LaunchPanel->cancel_button, 156, 202);
  WMSetButtonText (LaunchPanel->cancel_button, "Cancel");
  WMSetButtonAction (LaunchPanel->cancel_button, (WMAction *) LaunchAbortAction,
					 NULL);

  /* List widget */
  LaunchPanel->varlist = WMCreateList (LaunchPanel->window);
  WMSetListDoubleAction (LaunchPanel->varlist, (WMAction *) ListEnteredAction, NULL);
  WMResizeWidget (LaunchPanel->varlist, 308,
				  WMFontHeight ((wm_screen->normalFont)) * 7 + 14);
  WMMoveWidget (LaunchPanel->varlist, 8, 182);

  /* handlers */
  /*	WMCreateEventHandler (WMWidgetView (LaunchPanel->window), KeyPressMask,
		KeyPressAction, LaunchPanel->window);*/
  WMCreateEventHandler (WMWidgetView (LaunchPanel->command_tf), KeyPressMask,
						TextfieldPressAction, LaunchPanel->command_tf);
  WMCreateEventHandler (WMWidgetView (LaunchPanel->username_tf), KeyPressMask,
						UsernamePressAction, LaunchPanel->username_tf);
  WMCreateEventHandler (WMWidgetView (LaunchPanel->varlist), KeyPressMask,
						VarlistPressAction, LaunchPanel->varlist);
	
  WMRealizeWidget (LaunchPanel->window);
  WMSetFocusToWidget (LaunchPanel->command_tf);
}
