/*
 *	Info Panel
 */

#include "launch.h"

static int InfoPanelRunning = 0;

typedef struct
{
  WMWindow *window;
  WMLabel  *icon_label;
  WMPixmap *icon;
  WMFont   *title_font;
  WMFont   *copyright_font;
  WMLabel  *title_label;
  WMFrame  *hr_line;
  WMLabel  *info_label;
  WMLabel  *info_label1;
  WMLabel  *copyright_label;
}_InfoPanel;

static _InfoPanel	*InfoPanel;

/***************************************/

static void
closeInfoPanel (void)
{
  WMUnmapSubwidgets (InfoPanel->window);
  WMUnmapWidget (InfoPanel->window);
  WMDestroyWidget (InfoPanel->window);
  InfoPanelRunning = 0;
  free (InfoPanel);
}

void
runInfoPanel (void)
{
  int x, y;

  if (InfoPanelRunning == 1)
	return;
  InfoPanelRunning = 1;

  InfoPanel = (_InfoPanel *) malloc (sizeof (_InfoPanel));

  InfoPanel->window = WMCreatePanelWithStyleForWindow (LaunchPanel->window,
													   "Info", 
													   WMClosableWindowMask 
													   | WMTitledWindowMask);
  WMSetWindowTitle (InfoPanel->window, "Info");
  WMResizeWidget (InfoPanel->window, 420, 180);
  WMSetWindowMaxSize (InfoPanel->window, 420, 180);
  WMSetWindowMinSize (InfoPanel->window, 420, 180);
  WMSetWindowCloseAction (InfoPanel->window, (WMAction *) closeInfoPanel, NULL);
  WMSetWindowLevel(InfoPanel->window, WMModalPanelWindowLevel);

  x = (WidthOfScreen (DefaultScreenOfDisplay (display)) -
	   WMWidgetWidth (InfoPanel->window)) / 2;
  y = (HeightOfScreen (DefaultScreenOfDisplay (display)) -
	   WMWidgetHeight (InfoPanel->window)) / 2;
  WMSetWindowInitialPosition (InfoPanel->window, x, y);
  WMMoveWidget (InfoPanel->window, x, y);

  InfoPanel->icon = 
	SMCreateBlendedPixmapFromFile (wm_screen, PREFIX "/Launch.app/Launch.tiff");
  InfoPanel->icon_label = WMCreateLabel (InfoPanel->window);
  WMMoveWidget (InfoPanel->icon_label, 10, 10);
  WMResizeWidget (InfoPanel->icon_label, 48, 48);
  WMSetLabelImage (InfoPanel->icon_label, InfoPanel->icon);
  WMSetLabelImagePosition (InfoPanel->icon_label, WIPImageOnly);
  WMReleasePixmap (InfoPanel->icon);

  InfoPanel->icon = 
	SMCreateBlendedPixmapFromFile (wm_screen, PREFIX "/Launch.app/LaunchInfo.tiff");
  InfoPanel->title_label = WMCreateLabel (InfoPanel->window);
  WMResizeWidget (InfoPanel->title_label, 342, 48);
  WMMoveWidget (InfoPanel->title_label, 68, 14);
  WMSetLabelImage (InfoPanel->title_label, InfoPanel->icon);
  WMSetLabelImagePosition (InfoPanel->title_label, WIPImageOnly);
  WMReleasePixmap (InfoPanel->icon);

  InfoPanel->hr_line = WMCreateFrame (InfoPanel->window);
  WMResizeWidget (InfoPanel->hr_line, 342, 2);
  WMMoveWidget (InfoPanel->hr_line, 68, 68);

  InfoPanel->title_font = 
	WMCreateFont (wm_screen,"-*-helvetica-bold-r-*-*-*-140-*-*-*-*-*-*");
  InfoPanel->info_label = WMCreateLabel (InfoPanel->window);
  WMResizeWidget (InfoPanel->info_label, 342, 14);
  WMMoveWidget (InfoPanel->info_label, 68, 80);
  WMSetLabelText (InfoPanel->info_label, "Application launcher");
  WMSetLabelFont (InfoPanel->info_label, InfoPanel->title_font);
  WMSetLabelTextAlignment (InfoPanel->info_label, WACenter);

  InfoPanel->info_label = WMCreateLabel (InfoPanel->window);
  WMResizeWidget (InfoPanel->info_label, 342, 14);
  WMMoveWidget (InfoPanel->info_label, 68, 98);
  WMSetLabelText (InfoPanel->info_label, "Version " VERSION);
  WMSetLabelTextAlignment (InfoPanel->info_label, WACenter);

  InfoPanel->copyright_font = 
	WMCreateFont (wm_screen, "-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*");
  InfoPanel->copyright_label = WMCreateLabel (InfoPanel->window);
  WMResizeWidget (InfoPanel->copyright_label, 342, 12);
  WMSetLabelFont (InfoPanel->copyright_label, InfoPanel->copyright_font);
  WMMoveWidget (InfoPanel->copyright_label, 68, 145);
  WMSetLabelTextAlignment (InfoPanel->copyright_label, WACenter);
  WMSetLabelTextColor(InfoPanel->copyright_label, WMDarkGrayColor(wm_screen));
  WMSetLabelText (InfoPanel->copyright_label,
				  "Copyright (c) 2000-2002 by Serg Stoyan <stoyan@on.com.ua>");

  InfoPanel->copyright_label = WMCreateLabel (InfoPanel->window);
  WMResizeWidget (InfoPanel->copyright_label, 342, 12);
  WMSetLabelFont (InfoPanel->copyright_label, InfoPanel->copyright_font);
  WMMoveWidget (InfoPanel->copyright_label, 68, 160);
  WMSetLabelTextAlignment (InfoPanel->copyright_label, WACenter);
  WMSetLabelTextColor(InfoPanel->copyright_label, WMDarkGrayColor(wm_screen));
  WMSetLabelText (InfoPanel->copyright_label,
				  "Copyright (c) 2001 by Andy Ramensky <andy@hologr.com>");

  WMRealizeWidget (InfoPanel->window);
  WMMapSubwidgets (InfoPanel->window);
  WMMapWidget (InfoPanel->window);
}
