/*
 *	Launch application.
 *	Functions for creating and destroying aplication
 */

#include "launch.h"

static void LaunchCleanUp (void);
void        LaunchAbortAction (void);
void        LaunchCloseAction (void);
char        *KeyPressAction (XEvent * event, WMWidget *widget);
static void createMenu (void);

/********************************* Callbacks section **************************/

/**/
static void
LaunchCleanUp (void)
{
  int i;
  
  WMUnmapSubwidgets (LaunchPanel->window);
  WMUnmapWidget (LaunchPanel->window);
  
  for (i = 0; i < COMMANDS_NUMBER; i++)
  {
    free (command_history[i]);
  }
  free (entered_text);
  free (entered_user);
  free (leader_text);
  free (leader_tmp);
  free (complete_text);
	free (home_var);
  free (path);
  free (history_path);
}

/**/
void
LaunchAbortAction (void)
{
  LaunchCleanUp ();
  exit (0);
}

/**/
void
LaunchCloseAction (void)
{
  int prev_comm = 0;
  
  strcpy (command_history[commands_count], entered_text);
  
  if (commands_count)
    prev_comm = commands_count - 1;
  
  if (strcmp (entered_text, "")
      && strcmp (command_history[prev_comm], entered_text)
      && (commands_count != 0) && !su)
  {
    /* shift commands' stack in history file & add the new one */
    if (commands_count == COMMANDS_NUMBER)
    {
      current_command = 1;
      while (current_command <= COMMANDS_NUMBER)
      {
        strcpy (command_history[current_command - 1],
            command_history[current_command]);
        current_command++;
      }
      commands_count--;
    }
    saveHistory ();
  }
  if (commands_count == 0)
    saveHistory ();
  
  if (strcmp (entered_text, ""))
  {
    runCommand ();
  }
  LaunchCleanUp ();
  exit (0);
}

/**/
char *
KeyPressAction (XEvent * event, WMWidget *widget)
{
  char	*key_pressed;
  
  key_pressed = strdup (XKeysymToString (XLookupKeysym (&event->xkey, 0)));
  if (event->xkey.state == ShiftMask)
  {
    strcat(key_pressed, "Shifted");
  }
  
  if (auth != 1)
    WMSetLabelText (LaunchPanel->info, "Use Tab key to command completion.");
  
  /* Intercept? */
  if (!strcmp (key_pressed, "q") && event->xkey.state == Mod1Mask)
  {
    LaunchAbortAction ();
  }
  if (!strcmp (key_pressed, "Escape") && widget != LaunchPanel->varlist)
  {
    LaunchAbortAction ();
  }
  
  return key_pressed;
}

/* Application Menu */
static void
createMenu (void)
{
  mainmenu = WMMenuCreate (app, "Launch");
  info_submenu = WMMenuCreate (app, "Info");
  edit_submenu = WMMenuCreate (app, "Edit");
  windows_submenu = WMMenuCreate (app, "Windows");
  services_submenu = WMMenuCreate (app, "Services");
  
  WMMenuAddSubmenu (mainmenu, "Info", info_submenu);
  WMMenuAddItem (info_submenu, "Info panel...",
      (WMMenuAction) runInfoPanel, NULL, NULL, NULL);
  WMMenuAddItem (info_submenu, "Legal...",
      (WMMenuAction) runLegalPanel, NULL, NULL, NULL);
  WMMenuAddItem (info_submenu, "Preferences...",
      (WMMenuAction) runPreferencesPanel, NULL, NULL, NULL);
  
  WMMenuAddItem (mainmenu, "Hide", (WMMenuAction) WMHideApplication, app,
      NULL, "h");
  WMMenuAddItem (mainmenu, "Quit", (WMMenuAction) LaunchAbortAction, 
      NULL, NULL, "q");
  WMAppSetMainMenu (app, mainmenu);
  WMRealizeMenus (app);
}

/*********************************** MAIN *************************************/
int
main (int argc, char **argv)
{
  int           i, menuEnl = 1, parErr = 0;

  auth = 0;
  auth_result = 0;
  su = 0;
  tabkeypress = 0;
  setlocale (LC_ALL, "");
  entered_text = (char *) malloc (256);
  entered_user = (char *) malloc (256);
  leader_text = (char *) malloc (256);
  leader_tmp = (char *) malloc (256);
  complete_text = (char *) malloc (256);
  home_var = (char *) malloc (63);
  path = (char *) malloc (1024);
  history_path = (char *) malloc (256);
  for (i = 0; i < 255; i++)
  {
    entered_text[i] = '\0';
  }
  entered_text[i + 1] = '\0';
	
  strcpy (home_var, getenv ("HOME"));
  strcpy (path, getenv ("PATH"));
  strcpy (history_path, home_var);
  strcat (history_path, "/GNUstep/.AppInfo/Launch/History");

  /* loading history file */
  loadHistory ();

  LaunchPanel = (_LaunchPanel *) malloc (sizeof (_LaunchPanel));

  WMInitializeApplication ("Launch", &argc, argv);

  if (!(display = XOpenDisplay ("")))
  {
    fprintf (stderr, "Could not open Display!\n");
    LaunchAbortAction ();
  }
  
  /* Get preferences */
  udb = WMGetStandardUserDefaults ();

  if ((WMGetUDStringForKey (udb, "Version") == NULL))
  {
    WMSetUDStringForKey (udb, "Launch", "Application");
    WMSynchronizeUserDefaults (udb);
    WMSetUDStringForKey (udb, "0.65", "Version");
    WMSynchronizeUserDefaults (udb);
    WMSetUDStringForKey (udb,
        "/usr/local/GNUstep/Apps/Launch.app/openurl %s", "HTTP_App");
		WMSetUDStringForKey (udb,
        "/usr/local/GNUstep/Apps/Launch.app/openurl %s", "FTP_App");
    WMSetUDStringForKey (udb, "xterm -e %s", "Term_Com");
    WMSetUDBoolForKey (udb, True, "Enable_Switch");
    WMSetUDBoolForKey (udb, False, "PreserveEnvironment");
    WMSynchronizeUserDefaults (udb);
    WMSetUDStringForKey (udb, "20", "CommandsToRemember");
    WMSynchronizeUserDefaults (udb);
    WMSaveUserDefaults (udb);
  }
  else
  {
    if ((strcmp (WMGetUDStringForKey (udb, "Version"), "0.65")) != 0)
    {
      WMSetUDStringForKey (udb, "0.65", "Version");
      WMSynchronizeUserDefaults (udb);
      WMSetUDStringForKey (udb,
          "/usr/local/GNUstep/Apps/Launch.app/openurl %s", "HTTP_App");
      WMSetUDStringForKey (udb,
          "/usr/local/GNUstep/Apps/Launch.app/openurl %s", "FTP_App");
      WMSynchronizeUserDefaults (udb);
      WMSetUDStringForKey (udb, "20", "CommandsToRemember");
      WMSynchronizeUserDefaults (udb);
      WMSaveUserDefaults (udb);
    }
  }
  
  wm_screen = WMCreateScreen (display, DefaultScreen (display));
  createCommonPanel ();
  
  if (argc > 1)
  {
    if (!strcmp (argv[1], "-m"))
    {
      menuEnl = 0;
      if (argc > 2)
      {
        if (!strcmp (argv[2], "-su") && argc >= 5)
          su = 2;
        else
          parErr = 1;
      }
    }
    else
    {
      if (!strcmp (argv[1], "-su") && argc >= 4)
        su = 1;
      else
        parErr = 1;
    }
  }
  
  if (parErr)
  {
    fprintf (stderr, "Usage:\n\tLaunch [-m] [-su username command]\n ");
    LaunchAbortAction ();
  }
  
  if (su)
  {
    auth = 1;
    strcpy (entered_user, argv[su+1]);
    for (i = su+2; i < argc; i++)
    {
      strcat (entered_text, argv[i]);
      strcat (entered_text, " ");
    }
    su = 1;
    if (menuEnl)
      createMenu ();
    runAuthPanel ();
  }
  else
  {
    if (menuEnl)
      createMenu ();
    runExecPanel ();
  }
  
  /* setting window manager hints */
  configureCommonPanel ();
  
  WMSetApplicationIconPixmap (wm_screen, LaunchPanel->icon_pixmap);
  WMSetWindowMiniwindowPixmap (LaunchPanel->window, LaunchPanel->icon_pixmap);
  
  WMMapSubwidgets (LaunchPanel->window);
  if (!su)
  {
    WMUnmapWidget (LaunchPanel->varlist);
  }

  WMMapWidget (LaunchPanel->window);
  
  while (1)
  {
    XEvent event;
    
    WMNextEvent (display, &event);
    WMHandleEvent (&event);
    WMProcessEvent (app, &event);
  }
  
  exit (0);
}
