/*
 *	Preferences Panel
 */

#include "launch.h"

int	PreferencesPanelRunning = 0;

typedef struct
{
  WMWindow    *window;
  WMFrame     *app_frame;
  WMLabel     *info;
  WMLabel     *info_http;
  WMTextField *text_http;
  WMLabel     *info_ftp;
  WMTextField *text_ftp;
  WMLabel     *info_terminal;
  WMTextField *text_terminal;
  WMFrame     *console_frame;
  WMButton    *check_console;
  WMFrame     *history_frame;
  WMLabel     *commands_label;
  WMTextField *commands_tf;
  WMFrame     *env_frame;
  WMButton    *env_preserve;
  WMButton    *save_button;
  WMButton    *cancel_button;
} _PreferencesPanel;

_PreferencesPanel	*PreferencesPanel;

/* actions ******************************************************************/

/*
 *	Close preferences window
 */
static void
CloseAction (void)
{
  WMDestroyWidget (PreferencesPanel->window);

  free (PreferencesPanel);
  PreferencesPanelRunning = 0;

  if (WMGetUDBoolForKey (udb, "Enable_Switch") && !ConsoleSwitchVisible)
	{
	  LaunchPanel->switch_button = WMCreateSwitchButton (LaunchPanel->window);
	  WMResizeWidget (LaunchPanel->switch_button, 140, 20);
	  WMMoveWidget (LaunchPanel->switch_button, 12, 206);
	  WMSetButtonText (LaunchPanel->switch_button, "Console application");
	  WMSetButtonSelected (LaunchPanel->switch_button, False);
	  ConsoleSwitchVisible = 1;
	}

  if (!(WMGetUDBoolForKey (udb, "Enable_Switch")) && ConsoleSwitchVisible)
	{
	  WMDestroyWidget (LaunchPanel->switch_button);
	  ConsoleSwitchVisible = 0;
	}
  WMRealizeWidget (LaunchPanel->window);

  if (WMGetUDBoolForKey (udb, "Enable_Switch"))
	{
	  WMMapWidget (LaunchPanel->switch_button);
	}

  /* Cancel modal panel behaviour */
  WMBreakModalLoop (wm_screen);
}

/*
 *	Save preferences and exit
 */
static void
SaveAction (void)
{
  WMSetUDStringForKey (udb, WMGetTextFieldText (PreferencesPanel->text_http),
					   "HTTP_App");
  WMSetUDStringForKey (udb, WMGetTextFieldText (PreferencesPanel->text_ftp),
					   "FTP_App");
  WMSetUDStringForKey (udb, 
					   WMGetTextFieldText (PreferencesPanel->text_terminal), "Term_Com");
  WMSetUDBoolForKey (udb, WMGetButtonSelected (PreferencesPanel->check_console),
					 "Enable_Switch");
  WMSetUDBoolForKey (udb, WMGetButtonSelected (PreferencesPanel->env_preserve),
					 "PreserveEnvironment");
  WMSaveUserDefaults (udb);
  WMSynchronizeUserDefaults (udb);
  CloseAction ();
}

void
KeyPressActionPref (XEvent * event, WMWidget * widget)
{
  char *key_pressed = "";

  key_pressed = XKeysymToString (XLookupKeysym (&event->xkey, 0));

  if (!strcmp (key_pressed, "Tab"))
	{
	  if (widget == PreferencesPanel->text_http)
		{
		  if (event->xkey.state == ShiftMask)
			WMSetFocusToWidget (PreferencesPanel->text_terminal);
		  else
			WMSetFocusToWidget (PreferencesPanel->text_ftp);
		}
	  else
		{
		  if (widget == PreferencesPanel->text_ftp)
			{
			  if (event->xkey.state == ShiftMask)
				WMSetFocusToWidget (PreferencesPanel->text_http);
			  else
				WMSetFocusToWidget (PreferencesPanel->text_terminal);
			}
		  else
			{
			  if (widget == PreferencesPanel->text_terminal)
				{
				  if (event->xkey.state == ShiftMask)
					WMSetFocusToWidget (PreferencesPanel->text_ftp);
				  else
					WMSetFocusToWidget (PreferencesPanel->text_http);
				}
			}
		}
	}
	
  if (!strcmp (key_pressed, "Return") || !strcmp (key_pressed, "KP_Enter"))
	{
	  SaveAction ();
	}

  if (!strcmp (key_pressed, "Escape"))
	{
	  CloseAction ();
	}
}

/******************************************************************************/
void
runPreferencesPanel (void)
{
  if (PreferencesPanelRunning)
	{
	  return;
	}
  PreferencesPanelRunning = 1;

  PreferencesPanel = (_PreferencesPanel *) malloc (sizeof (_PreferencesPanel));

  PreferencesPanel->window = 
	WMCreatePanelWithStyleForWindow (LaunchPanel->window, "Launch Preferences",
									 WMTitledWindowMask | WMClosableWindowMask);
  WMResizeWidget (PreferencesPanel->window, 324, 340);
  WMSetWindowUserPosition (PreferencesPanel->window, 100, 0);
  WMSetWindowTitle (PreferencesPanel->window, "Launch Preferences");
  WMSetWindowCloseAction (PreferencesPanel->window, (WMAction *) CloseAction,
						  NULL);
  WMSetWindowLevel(PreferencesPanel->window, WMModalPanelWindowLevel);

  /* Frame "URL helpers" ***********************************************/
  PreferencesPanel->app_frame = WMCreateFrame (PreferencesPanel->window);
  WMMoveWidget (PreferencesPanel->app_frame, 10, 10);
  WMResizeWidget (PreferencesPanel->app_frame, 304, 76);
  WMSetFrameTitle (PreferencesPanel->app_frame, " URL helpers ");

  /* HTTP label */
  PreferencesPanel->info_http = WMCreateLabel (PreferencesPanel->app_frame);
  WMResizeWidget (PreferencesPanel->info_http, 60, 14);
  WMMoveWidget (PreferencesPanel->info_http, 3, 26);
  WMSetLabelTextAlignment (PreferencesPanel->info_http, WALeft);
  WMSetLabelText (PreferencesPanel->info_http, "WWW:");

  /* HTTP textfield */
  PreferencesPanel->text_http = WMCreateTextField (PreferencesPanel->app_frame);
  WMResizeWidget (PreferencesPanel->text_http, 236, 20);
  WMMoveWidget (PreferencesPanel->text_http, 60, 22);
  WMSetTextFieldText (PreferencesPanel->text_http,
					  WMGetUDStringForKey (udb, "HTTP_App"));
  WMSetTextFieldEditable (PreferencesPanel->text_http, True);

  /* FTP label */
  PreferencesPanel->info_ftp = WMCreateLabel (PreferencesPanel->app_frame);
  WMResizeWidget (PreferencesPanel->info_ftp, 60, 14);
  WMMoveWidget (PreferencesPanel->info_ftp, 3, 52);
  WMSetLabelTextAlignment (PreferencesPanel->info_ftp, WALeft);
  WMSetLabelText (PreferencesPanel->info_ftp, "FTP:");

  /* FTP textfield */
  PreferencesPanel->text_ftp = WMCreateTextField (PreferencesPanel->app_frame);
  WMResizeWidget (PreferencesPanel->text_ftp, 236, 20);
  WMMoveWidget (PreferencesPanel->text_ftp, 60, 48);
  WMSetTextFieldText (PreferencesPanel->text_ftp,
					  WMGetUDStringForKey (udb, "FTP_App"));
  WMSetTextFieldEditable (PreferencesPanel->text_ftp, True);

  /* Frame "Console" ***************************************************/
  PreferencesPanel->console_frame = WMCreateFrame (PreferencesPanel->window);
  WMResizeWidget (PreferencesPanel->console_frame, 304, 76);
  WMMoveWidget (PreferencesPanel->console_frame, 10, 96);
  WMSetFrameTitle (PreferencesPanel->console_frame, " Console ");

  /* Terminal label */
  PreferencesPanel->info_terminal = 
    WMCreateLabel(PreferencesPanel->console_frame);
  WMResizeWidget (PreferencesPanel->info_terminal, 60, 14);
  WMMoveWidget (PreferencesPanel->info_terminal, 3, 28);
  WMSetLabelTextAlignment (PreferencesPanel->info_terminal, WALeft);
  WMSetLabelText (PreferencesPanel->info_terminal, "Terminal:");

  /* Terminal textfield */
  PreferencesPanel->text_terminal = 
    WMCreateTextField (PreferencesPanel->console_frame);
  WMResizeWidget (PreferencesPanel->text_terminal, 236, 20);
  WMMoveWidget (PreferencesPanel->text_terminal, 60, 24);
  WMSetTextFieldText (PreferencesPanel->text_terminal,
					  WMGetUDStringForKey (udb, "Term_Com"));
  WMSetTextFieldEditable (PreferencesPanel->text_terminal, True);

  /* "Console app" checkbox */
  PreferencesPanel->check_console = 
	WMCreateSwitchButton (PreferencesPanel->console_frame);
  WMResizeWidget (PreferencesPanel->check_console, 284, 20);
  WMMoveWidget (PreferencesPanel->check_console, 8, 50);
  WMSetButtonText (PreferencesPanel->check_console,
				   "Show \"Console application\" switch");
  WMSetButtonSelected (PreferencesPanel->check_console,
					   (WMGetUDBoolForKey (udb, "Enable_Switch")));

  /* Frame "History" ***************************************************/
  PreferencesPanel->history_frame = WMCreateFrame (PreferencesPanel->window);
  WMResizeWidget (PreferencesPanel->history_frame, 304, 50);
  WMMoveWidget (PreferencesPanel->history_frame, 10, 184);
  WMSetFrameTitle (PreferencesPanel->history_frame, " History ");

  /* Commands label */
  PreferencesPanel->commands_label = 
    WMCreateLabel(PreferencesPanel->history_frame);
  WMResizeWidget (PreferencesPanel->commands_label,
				  WMWidthOfString(WMSystemFontOfSize(wm_screen, 12), 
								  "Commands to remember",
								  strlen("Commands to remember: ")), 14);
  WMMoveWidget (PreferencesPanel->commands_label, 3, 22);
  WMSetLabelTextAlignment (PreferencesPanel->commands_label, WALeft);
  WMSetLabelText (PreferencesPanel->commands_label, "Commands to remember:");

  /* Commands textfield */
  PreferencesPanel->commands_tf = 
    WMCreateTextField (PreferencesPanel->history_frame);
  WMResizeWidget (PreferencesPanel->commands_tf, 40, 20);
  WMMoveWidget (PreferencesPanel->commands_tf,
				WMWidthOfString(WMSystemFontOfSize(wm_screen, 12), 
								"Commands to remember",
								strlen("Commands to remember: "))+3, 18);
  WMSetTextFieldText (PreferencesPanel->commands_tf,
					  WMGetUDStringForKey (udb, "CommandsToRemember"));
  WMSetTextFieldEditable (PreferencesPanel->commands_tf, True);

  /* Frame "Environment" ***************************************************/
  PreferencesPanel->env_frame = WMCreateFrame (PreferencesPanel->window);
  WMResizeWidget (PreferencesPanel->env_frame, 304, 50);
  WMMoveWidget (PreferencesPanel->env_frame, 10, 244);
  WMSetFrameTitle (PreferencesPanel->env_frame, " Environment ");

  PreferencesPanel->env_preserve = 
	WMCreateSwitchButton (PreferencesPanel->env_frame);
  WMResizeWidget (PreferencesPanel->env_preserve, 284, 20);
  WMMoveWidget (PreferencesPanel->env_preserve, 8, 20);
  WMSetButtonText (PreferencesPanel->env_preserve,
				   "Preserve current user environment");
  WMSetButtonSelected (PreferencesPanel->env_preserve,
					   (WMGetUDBoolForKey (udb, "PreserveEnvironment")));

  /* buttons widgets ***************************************************/
  /* "Save" */
  PreferencesPanel->save_button =
	WMCreateCommandButton (PreferencesPanel->window);
  WMSetButtonAction (PreferencesPanel->save_button, (WMAction *) SaveAction,
					 NULL);
  WMResizeWidget (PreferencesPanel->save_button, 72, 24);
  WMMoveWidget (PreferencesPanel->save_button, 
				WMWidgetWidth(PreferencesPanel->window) - (72 + 10), 306);
  WMSetButtonText (PreferencesPanel->save_button, "Save");
  WMSetButtonImage (PreferencesPanel->save_button,
					WMGetSystemPixmap (wm_screen, WSIReturnArrow));
  WMSetButtonAltImage (PreferencesPanel->save_button,
					   WMGetSystemPixmap (wm_screen, WSIHighlightedReturnArrow));
  WMSetButtonImagePosition (PreferencesPanel->save_button, WIPRight);
  WMSetButtonAction (PreferencesPanel->save_button, (WMAction *) SaveAction,
					 NULL);

  /* "Cancel" */
  PreferencesPanel->cancel_button =
	WMCreateCommandButton (PreferencesPanel->window);
  WMResizeWidget (PreferencesPanel->cancel_button, 72, 24);
  WMMoveWidget (PreferencesPanel->cancel_button,
				WMWidgetWidth(PreferencesPanel->window) - (72*2 + 10 + 8), 306);
  WMSetButtonText (PreferencesPanel->cancel_button, "Cancel");
  WMSetButtonAction (PreferencesPanel->cancel_button,
					 (WMAction *) CloseAction, NULL);

  /* handlers */
  WMCreateEventHandler (WMWidgetView (PreferencesPanel->text_http),
						KeyPressMask, KeyPressActionPref,
						PreferencesPanel->text_http);
  WMCreateEventHandler (WMWidgetView (PreferencesPanel->text_ftp),
						KeyPressMask, KeyPressActionPref,
						PreferencesPanel->text_ftp);
  WMCreateEventHandler (WMWidgetView (PreferencesPanel->text_terminal),
						KeyPressMask, KeyPressActionPref,
						PreferencesPanel->text_terminal);
  WMRealizeWidget (PreferencesPanel->window);

  WMAppAddWindow (app, WMWidgetXID (PreferencesPanel->window));
  WMMapSubwidgets (PreferencesPanel->app_frame);
  WMMapSubwidgets (PreferencesPanel->console_frame);
  WMMapSubwidgets (PreferencesPanel->history_frame);
  WMMapSubwidgets (PreferencesPanel->env_frame);
  WMMapSubwidgets (PreferencesPanel->window);
  WMMapWidget (PreferencesPanel->window);
  WMSetFocusToWidget (PreferencesPanel->text_http);
  WMMapWidget (PreferencesPanel->window);

  /* Loop for true modal panel behaviour */
  WMRunModalLoop (wm_screen, WMWidgetView (PreferencesPanel->window));
}
