/*
 *	"User Authentication" panel and auth related functions
 */

#include "launch.h"
#include <crypt.h>

#ifdef USE_SHADOW
#include <shadow.h>
#endif

/*
 *	Catch pressing Enter key event
 */
void
AuthPasswordPressAction (XEvent * event, WMWidget *widget)
{
  char	*key_pressed;
  
  key_pressed = KeyPressAction(event, widget);

  if (!strcmp (key_pressed, "Return") || !strcmp (key_pressed, "KP_Enter"))
	{
	  authOkAction ();
	}

  return;
}

/*
 *	Check username and password
 */
int
authUser (void)
{
  char				*password = NULL;
  char				*encrypted_password = NULL;
  char				*correct_password = NULL;
#ifdef USE_SHADOW
  struct spwd	*spwd;
#endif

  password = (char *)malloc(128);

  strcpy (password, WMGetTextFieldText (LaunchPanel->auth_password));

  if ((pw = getpwnam (entered_user)) == NULL)
	{
	  /* username incorrect. */
	  return 2;			
	}
#ifdef USE_SHADOW
  spwd = getspnam (entered_user);
  if (spwd)
	correct_password = spwd->sp_pwdp;
#else
  correct_password = pw->pw_passwd;
#endif

  encrypted_password = (char *) crypt (password, correct_password);
  if (strcmp (encrypted_password, correct_password))
	{
	  /* password incorrect */
	  return 3;			
	}

  /* user authenticated successfully. */
  return 1;			
}

/*
 *	Authenticate? Yes, sir!
 */
void
authOkAction (void)
{
  auth_result = authUser ();
  if (auth_result == 1)
	{
	  LaunchCloseAction ();
	}
  else
	{
	  if (auth_result == 2)
		SMRunAttentionPanel (wm_screen, LaunchPanel->window, 
							 "User Authentication error",
							 "You've entered incorrect username", NULL, NULL, "Ok");
	  if (auth_result == 3)
		SMRunAttentionPanel (wm_screen, LaunchPanel->window, 
							 "User authentication error",
							 "You've entered incorrect password", NULL, NULL, "Ok");
	  LaunchAbortAction ();
	}
}

/*
 *	Panel
 */
void
runAuthPanel (void)
{
  if (!su)
	{
	  WMUnmapWidget (LaunchPanel->username_tf);
	  WMUnmapWidget (LaunchPanel->username_prompt);
	  WMUnmapWidget (LaunchPanel->command_tf);
	  WMUnmapWidget (LaunchPanel->command_prompt);
	  WMUnmapWidget (LaunchPanel->run_button);
	  WMUnmapWidget (LaunchPanel->cancel_button);
	  if (ConsoleSwitchVisible)
		WMUnmapWidget (LaunchPanel->switch_button);
	  WMUnmapWidget (LaunchPanel->varlist);
	}

  WMResizeWidget (LaunchPanel->window, 324, 214);

  WMSetLabelText (LaunchPanel->title, "User Authentication");

  /* info label */
  WMSetLabelText (LaunchPanel->info, "You have to enter password for user ");

  /* user info label */
  LaunchPanel->info_user = WMCreateLabel (LaunchPanel->window);
  WMSetLabelText (LaunchPanel->info_user, entered_user);
  WMMoveWidget (LaunchPanel->info_user, 0, 115);
  WMResizeWidget (LaunchPanel->info_user, 324, 20);
  WMSetLabelFont (LaunchPanel->info_user, LaunchPanel->title_font);
  WMSetLabelTextAlignment (LaunchPanel->info_user, WACenter);

  /* password prompt */
  LaunchPanel->auth_prompt = WMCreateLabel (LaunchPanel->window);
  WMSetLabelText (LaunchPanel->auth_prompt, "Password:");
  WMMoveWidget (LaunchPanel->auth_prompt, 12, 156);
  WMResizeWidget (LaunchPanel->auth_prompt, 304, 12);

  /* password textfield */
  LaunchPanel->auth_password = WMCreateTextField (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->auth_password, 240, 20);
  WMMoveWidget (LaunchPanel->auth_password, 76, 154);
  WMSetTextFieldSecure (LaunchPanel->auth_password, True);

  /* "Ok" buttons */
  LaunchPanel->auth_ok_button = WMCreateCommandButton (LaunchPanel->window);
  WMSetButtonAction (LaunchPanel->auth_ok_button, (WMAction *) authOkAction, 
					 NULL);
  WMResizeWidget (LaunchPanel->auth_ok_button, 72, 24);
  WMMoveWidget (LaunchPanel->auth_ok_button, 244, 182);
  WMSetButtonText (LaunchPanel->auth_ok_button, "Login");
  WMSetButtonImage (LaunchPanel->auth_ok_button, 
					WMGetSystemPixmap (wm_screen, WSIReturnArrow));
  WMSetButtonAltImage (LaunchPanel->auth_ok_button, 
					   WMGetSystemPixmap (wm_screen, WSIHighlightedReturnArrow));
  WMSetButtonImagePosition (LaunchPanel->auth_ok_button, WIPRight);

  /* "Cancel" button */
  LaunchPanel->auth_cancel_button = WMCreateCommandButton (LaunchPanel->window);
  WMResizeWidget (LaunchPanel->auth_cancel_button, 72, 24);
  WMMoveWidget (LaunchPanel->auth_cancel_button, 164, 182);
  WMSetButtonText (LaunchPanel->auth_cancel_button, "Cancel");
  WMSetButtonAction (LaunchPanel->auth_cancel_button, 
					 (WMAction *) LaunchAbortAction, NULL);

  WMCreateEventHandler (WMWidgetView (LaunchPanel->auth_password), KeyPressMask,
						AuthPasswordPressAction, LaunchPanel->auth_password);

  WMRealizeWidget (LaunchPanel->window);

  if (su)
	{
	  WMMapWidget (LaunchPanel->title);
	  WMMapWidget (LaunchPanel->info);
	}

  WMMapWidget (LaunchPanel->info_user);
  WMMapWidget (LaunchPanel->auth_prompt);
  WMMapWidget (LaunchPanel->auth_password);
  WMMapWidget (LaunchPanel->auth_ok_button);
  WMMapWidget (LaunchPanel->auth_cancel_button);

  WMSetFocusToWidget (LaunchPanel->auth_password);
}
