/*
 * Various utility functions that might go into StepMaker library.
 */

#include <stdio.h>
#include <stdlib.h>
#include <WINGs/WINGs.h>
#include <WINGs/WINGsP.h>
#include <X11/keysym.h>
#include "utils.h"

WMPixmap*
SMCreateBlendedPixmapFromFile(WMScreen *scr, char *fileName)
{
  WMPixmap  *pixmap;
  RColor    color;
  char      *component = NULL;

  component = (char *)malloc(5);
  
  snprintf(component, 3, "%x", WMRedComponentOfColor(WMGrayColor(scr)));
  color.red = strtol(component, (char **)NULL, 16);
  snprintf(component, 3, "%x", WMGreenComponentOfColor(WMGrayColor(scr)));
  color.green = strtol(component, (char **)NULL, 16);
  snprintf(component, 3, "%x", WMBlueComponentOfColor(WMGrayColor(scr)));
  color.blue = strtol(component, (char **)NULL, 16);
  
  pixmap = WMCreateBlendedPixmapFromFile(scr, fileName, &color);
  
  free(component);
  return pixmap;
}


/*****************************************************************************
 * Panels.
 ****************************************************************************/
void
PanelsKeypress(XEvent *event, WMWidget *widget)
{
  KeySym  keysym;
  int     alt;

  keysym = XLookupKeysym(&event->xkey, 0);
  alt = event->xkey.state & Mod1Mask;

  switch (keysym)
	{
    case XK_Return:
      WMPerformButtonClick(APanel->right_button);
      break;
    default:
      break;
	}
}

/*
 * Attention panel.
 *----------------------------------------------------------------------------
 * "User Interface Guide":
 * - It has an empty title bar.
 * - It's closed by one or more buttons in its content area, not by a button
 *   in the title bar.
 * - It stays on-screen even when the application isn't active until dismissed
 *   by the user.
 * - It's the key window(modal) whenever the application is active.
 * - It's isolated in a tier above everything on the screen except 
 *   spring-loaded windows such as pop-up lists.
 *----------------------------------------------------------------------------
 * This panel is modal(no other window actions of the app will be proceeded
 * until this panel is opened). Returns the number of the pressed button. 
 * If button text is NULL, button don't maps.
 */
void
closeAttentionPanel(WMButton *self)
{
  WMUnmapSubwidgets (APanel->panel);
  WMUnmapWidget (APanel->panel);
  WMDestroyWidget (APanel->panel);

  if (self == APanel->left_button)
    APanel->pressed_button = 1;
  if (self == APanel->center_button)
    APanel->pressed_button = 2;
  if (self == APanel->right_button)
    APanel->pressed_button = 3;

  WMBreakModalLoop(APanel->app);
}

int
SMRunAttentionPanel(WMScreen *app, WMWindow *parent, 
                    char *side_text, char *question_text,
                    char *left_button_text, char* center_button_text,
                    char *right_button_text)
{
  int       x, y;

  APanel = (_APanel *)malloc(sizeof(_APanel));

  APanel->app = app;
  APanel->pressed_button = 0;

  APanel->panel = WMCreatePanelWithStyleForWindow (parent, "AttentionPanel",
												   WMTitledWindowMask);
  WMResizeWidget(APanel->panel, 364, 185);
  WMSetWindowTitle(APanel->panel, "");
  x = (WidthOfScreen (DefaultScreenOfDisplay (app->display)) -
	   WMWidgetWidth (APanel->panel)) / 2;
  y = (HeightOfScreen (DefaultScreenOfDisplay (app->display)) -
	   WMWidgetHeight (APanel->panel)) / 2;
  WMSetWindowUserPosition (APanel->panel, x, y/2);
  WMSetWindowLevel(APanel->panel, WMModalPanelWindowLevel);

  // Icon
  APanel->icon_label = WMCreateLabel (APanel->panel);
  WMMoveWidget (APanel->icon_label, 10, 10);
  WMResizeWidget (APanel->icon_label, 48, 48);
  WMSetLabelImage (APanel->icon_label, WMGetApplicationIconPixmap(app));
  WMSetLabelImagePosition (APanel->icon_label, WIPImageOnly);

  // Icon side text
  APanel->side_label = WMCreateLabel(APanel->panel);
  WMResizeWidget (APanel->side_label, 364 - (64+16), 48);
  WMMoveWidget(APanel->side_label, 64, 8);
  WMSetLabelFont(APanel->side_label, WMSystemFontOfSize(app, 18));
  WMSetLabelText(APanel->side_label, side_text);

  // Horizontal line
  APanel->h_line = WMCreateFrame(APanel->panel);
  WMResizeWidget(APanel->h_line, 364, 2);
  WMMoveWidget(APanel->h_line, 0, 64);

  // Question text
  APanel->question_label = WMCreateLabel(APanel->panel);
  WMResizeWidget(APanel->question_label, 364 - 16, 84);
  WMMoveWidget(APanel->question_label, 8, 74);
  WMSetLabelText(APanel->question_label, question_text);
  WMSetLabelTextAlignment(APanel->question_label, WACenter);
  WMSetLabelFont(APanel->question_label, WMSystemFontOfSize(app, 14));
  WMSetLabelWraps(APanel->question_label, True);
  

  // Buttons
  if (left_button_text)
	{
	  APanel->left_button = WMCreateCommandButton(APanel->panel);
	  WMResizeWidget(APanel->left_button, 74, 24);
	  WMMoveWidget(APanel->left_button, 364 - (24+(74*3)), 185 - (24 + 8));
	  WMSetButtonText(APanel->left_button, left_button_text);
	  WMSetButtonAction(APanel->left_button, (WMAction *)closeAttentionPanel,
						APanel->left_button);
	}

  if (center_button_text)
	{
	  APanel->center_button = WMCreateCommandButton(APanel->panel);
	  WMResizeWidget(APanel->center_button, 74, 24);
	  WMMoveWidget(APanel->center_button, 364 - (16+(74*2)), 185 - (24 + 8));
	  WMSetButtonText(APanel->center_button, center_button_text);
	  WMSetButtonAction(APanel->center_button, (WMAction *)closeAttentionPanel,
						APanel->center_button);
	}

  APanel->right_button = WMCreateCommandButton(APanel->panel);
  WMResizeWidget(APanel->right_button, 74, 24);
  WMMoveWidget(APanel->right_button, 364 - (8+74), 185 - (24 + 8));
  if (right_button_text)
	{
	  WMSetButtonText(APanel->right_button, right_button_text);
	}
  else
	{
	  WMSetButtonText(APanel->right_button, "Close");
	}
  WMSetButtonImage (APanel->right_button,
					WMGetSystemPixmap (app, WSIReturnArrow));
  WMSetButtonAltImage (APanel->right_button, WMGetSystemPixmap (app,
																WSIHighlightedReturnArrow));
  WMSetButtonImagePosition (APanel->right_button, WIPRight);
  WMSetButtonAction(APanel->right_button, (WMAction *)closeAttentionPanel,
					APanel->right_button);

  WMCreateEventHandler(WMWidgetView(APanel->panel), KeyPressMask,
					   PanelsKeypress, APanel->panel);

  WMRealizeWidget(APanel->panel);
  WMMapSubwidgets(APanel->panel);
  WMMapWidget(APanel->panel);
  
  WMRunModalLoop (app, WMWidgetView (APanel->panel));

  return APanel->pressed_button;
}
